package ovh.herisson.Clyde.EndPoints.Inscription;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.CurriculumRepository;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.Inscription.InscriptionService;
import ovh.herisson.Clyde.Services.ProtectionService;
import ovh.herisson.Clyde.Tables.Curriculum;
import ovh.herisson.Clyde.Tables.Inscription.InscriptionRequest;
import ovh.herisson.Clyde.Tables.RequestState;
import ovh.herisson.Clyde.Tables.Role;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class InscriptionController {


    private final InscriptionService inscriptionServ;
    private final AuthenticatorService authServ;
    private final CurriculumRepository curriculumRepository;
    public InscriptionController(InscriptionService inscriptionServ, AuthenticatorService authServ, CurriculumRepository curriculumRepository){
        this.inscriptionServ = inscriptionServ;
        this.authServ = authServ;
        this.curriculumRepository = curriculumRepository;
    }


    @GetMapping("/requests/register")
    public ResponseEntity<Iterable<Map<String,Object>>> getAllRequests(@RequestHeader("Authorization") String token){

        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService, Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        Iterable<InscriptionRequest> inscriptionRequests = inscriptionServ.getAll();

        return new ResponseEntity<>(ProtectionService.requestsWithoutPasswords(inscriptionRequests), HttpStatus.OK);
    }


    @GetMapping("/request/register/{id}")
    public ResponseEntity<Map<String,Object>> getById(@RequestHeader("Authorization") String token, @PathVariable long id){

        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService, Role.Teacher},token))
            return new UnauthorizedResponse<>(null);

        InscriptionRequest foundInscriptionRequest = inscriptionServ.getById(id);

        if (foundInscriptionRequest == null)
            return new ResponseEntity<>(null, HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(ProtectionService.requestWithoutPassword(foundInscriptionRequest), HttpStatus.OK);
    }

    @PatchMapping("/request/register/{id}")
    public ResponseEntity<InscriptionRequest> changeRequestState(@PathVariable long id,
                                                                 @RequestHeader("Authorization") String token,
                                                                 @RequestBody RequestState state)
    {
        if (authServ.isNotIn(new Role[]{Role.InscriptionService,Role.Admin},token))
            return new UnauthorizedResponse<>(null);

        if (!inscriptionServ.modifyState(id, state))
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(HttpStatus.OK);
    }
    @DeleteMapping("/request/register/{id}")
    public ResponseEntity<String > deleteRequest(@RequestHeader("Authorization") String token, @PathVariable Long id){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.InscriptionService}, token))
            return new UnauthorizedResponse<>(null);

        InscriptionRequest toDelete = inscriptionServ.getById(id);

        if (toDelete == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        inscriptionServ.delete(toDelete);
        return new ResponseEntity<>(HttpStatus.OK);
    }


    //Allow teacher or admin to accept or refuse the equivalence
    @PatchMapping("/request/registerequiv/{id}/{newstate}")
    public ResponseEntity<Object> editRegisterEquiv(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable RequestState newstate){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher}, token))
            return new UnauthorizedResponse<>(null);

        InscriptionRequest toEdit = inscriptionServ.getById(id);

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (toEdit.getEquivalenceState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        toEdit.setEquivalenceState(newstate);

        inscriptionServ.save(toEdit);

        if (toEdit.getState() == RequestState.Accepted && (toEdit.getEquivalenceState() == RequestState.Accepted || toEdit.getEquivalenceState() == RequestState.Unrequired))
        {
            inscriptionServ.createUser(toEdit);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PatchMapping("/request/registerequivimpose/{id}/{cursusid}")
    public ResponseEntity<Object> editRegisterEquiv(@RequestHeader("Authorization") String token, @PathVariable long id, @PathVariable long cursusid){
        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Teacher}, token))
            return new UnauthorizedResponse<>(null);

        InscriptionRequest toEdit = inscriptionServ.getById(id);

        //If the request is already accepted we just return ok (otherwise we would duplicate the procedure below)
        if (toEdit.getEquivalenceState() == RequestState.Accepted){
            return new ResponseEntity<>(HttpStatus.OK);
        }

        //We impose a curriculum
        Curriculum curriculum = curriculumRepository.findById(cursusid);

        toEdit.setCurriculumId(curriculum.getCurriculumId());
        toEdit.setEquivalenceState(RequestState.Accepted);

        inscriptionServ.save(toEdit);

        if (toEdit.getState() == RequestState.Accepted && (toEdit.getEquivalenceState() == RequestState.Accepted || toEdit.getEquivalenceState() == RequestState.Unrequired))
        {
            inscriptionServ.createUser(toEdit);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }
}
